from __future__ import annotations

from typing import TYPE_CHECKING, TypeVar

if TYPE_CHECKING:
    from collections.abc import Mapping

K = TypeVar("K")
V = TypeVar("V")


def without_none_values(d: Mapping[K, V]) -> Mapping[K, V]:
    """Return a copy of d without None values."""
    return {k: v for k, v in d.items() if v is not None}



from __future__ import annotations

from typing import Literal

from pydantic import BaseModel

__all__ = ["Counts"]


class Counts(BaseModel):
    dataset_count: int
    id: Literal["_counts"] = "_counts"
    provider_count: int
    series_count: int



from __future__ import annotations

from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from dbnomics_data_model import DatasetCode, ProviderCode
    from solrq import Q

    from dbnomics_solr.types import SolrDoc

__all__ = [
    "DatasetAlreadyIndexed",
    "DatasetNotFound",
    "DBnomicsSolrException",
    "DuplicateDocuments",
    "IndexationError",
    "InvalidSolrDocument",
    "ProviderNotFound",
]


class DBnomicsSolrException(Exception):
    pass


class DatasetAlreadyIndexed(DBnomicsSolrException):
    def __init__(self, provider_code: ProviderCode, dataset_code: DatasetCode, *, dir_hash: str) -> None:
        dataset_id = f"{provider_code}/{dataset_code}"
        message = f"The dataset {dataset_id!r} is already indexed (dir_hash: {dir_hash})"
        super().__init__(message)
        self.provider_code = provider_code
        self.dataset_code = dataset_code
        self.dir_hash = dir_hash


class DatasetNotFound(DBnomicsSolrException):
    def __init__(self, provider_code: ProviderCode, dataset_code: DatasetCode) -> None:
        dataset_id = f"{provider_code}/{dataset_code}"
        message = f"The dataset {dataset_id!r} was not found"
        super().__init__(message)
        self.provider_code = provider_code
        self.dataset_code = dataset_code


class DuplicateDocuments(DBnomicsSolrException):
    def __init__(self, *, query: Q, hits: int) -> None:
        message = f"The query {query!r} was expected to return at most one document but it returned {hits}"
        super().__init__(message)
        self.query = query
        self.hits = hits


class InvalidSolrDocument(DBnomicsSolrException):
    def __init__(self, message: str = "Invalid Solr document", *, solr_document: SolrDoc) -> None:
        super().__init__(message)
        self.solr_document = solr_document


class ProviderNotFound(DBnomicsSolrException):
    def __init__(self, provider_slug: str) -> None:
        message = f"The provider {provider_slug!r} was not found"
        super().__init__(message)
        self.provider_slug = provider_slug


class IndexationError(DBnomicsSolrException):
    def __init__(self, error: dict[str, Any] | None = None) -> None:
        message = f"Indexation error: {error!r}"
        super().__init__(message)
        self.error = error



"""Domain-level service to update the counts of providers, datasets and series."""

from __future__ import annotations

from typing import TYPE_CHECKING

import daiquiri

if TYPE_CHECKING:
    from dbnomics_solr.dbnomics_solr_client import DBnomicsSolrClient

__all__ = ["update_top_level_counts"]


logger = daiquiri.getLogger(__name__)


def update_top_level_counts(*, commit: bool = True, dbnomics_solr_client: DBnomicsSolrClient) -> None:
    """Update the counts of providers, datasets and series."""
    logger.debug("Querying the top-level counts...")
    counts = dbnomics_solr_client.get_counts(force_count=True)

    logger.debug("Indexing the top-level counts: %r", counts)
    dbnomics_solr_client.index_counts(counts)

    if commit:
        dbnomics_solr_client.commit()

    logger.info("Top-level counts were updated: %r", counts)



from __future__ import annotations

from typing import TYPE_CHECKING, cast

from dirhash import dirhash

if TYPE_CHECKING:
    from pathlib import Path


def compute_dir_hash(dir_path: Path, /, *, jobs: int = 1) -> str:
    """Compute a hash for a directory in an opinionated way.

    Follows the [dirhash standard](https://github.com/andhus/dirhash).
    """
    return cast(str, dirhash(dir_path, algorithm="sha1", jobs=jobs))
